package legion.muyue.best2048;

import android.app.AlertDialog; // Ajout
import android.content.SharedPreferences; // Ajout
import android.os.Bundle;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater; // Ajout
import android.view.View; // Ajout
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.gridlayout.widget.GridLayout;
import android.widget.Button;


public class MainActivity extends AppCompatActivity {

    private GridLayout gameBoardLayout;
    private TextView scoreTextView;
    private TextView highScoreTextView;
    private Game game;
    private SharedPreferences sharedPreferences; // Ajout

    // Constantes SharedPreferences
    private static final String PREFS_NAME = "MyPrefs";
    private static final String GAME_STATE_KEY = "gameState";
    private static final String HIGH_SCORE_KEY = "highScore";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        EdgeToEdge.enable(this);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        gameBoardLayout = findViewById(R.id.gameBoard);
        scoreTextView = findViewById(R.id.scoreLabel);
        highScoreTextView = findViewById(R.id.highScoreLabel);
        Button restartButton = findViewById(R.id.restartButton);
        Button statsButton = findViewById(R.id.statsButton);
        Button menuButton = findViewById(R.id.menuButton);
        Button multiplayerButton = findViewById(R.id.multiplayerButton);

        // Initialisation SharedPreferences
        sharedPreferences = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);

        initGameBoardLayout();

        // Chargement du jeu (ou création si pas de sauvegarde)
        loadGame();

        setupSwipeListener();

        //Listeners des boutons
        multiplayerButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMultiplayerScreen();
        });
        // restartGame appelle maintenant la dialog
        restartButton.setOnClickListener(v -> restartGame());
        statsButton.setOnClickListener(v -> showStats());
        menuButton.setOnClickListener(v -> showMenu());
    }

    private void initGameBoardLayout() {
        gameBoardLayout.removeAllViews();
        gameBoardLayout.setColumnCount(4);
        gameBoardLayout.setRowCount(4);
    }

    private void updateUI() {
        gameBoardLayout.removeAllViews();

        for (int x = 0; x < 4; x++) {
            for (int y = 0; y < 4; y++) {
                TextView tileTextView = new TextView(this);
                int value = game.getGameBoard(x, y);

                setTileAppearance(tileTextView, value);

                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0;
                params.height = 0;
                params.rowSpec = GridLayout.spec(x, 1f);
                params.columnSpec = GridLayout.spec(y, 1f);
                params.setMargins(
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin)
                );
                tileTextView.setLayoutParams(params);

                gameBoardLayout.addView(tileTextView);
            }
        }
        scoreTextView.setText(getString(R.string.score_placeholder, game.getScore()));
        highScoreTextView.setText(getString(R.string.high_score_placeholder, game.getHighScore()));
    }

    private void setTileAppearance(TextView tile, int value) {
        int textColorId;
        int textSizeId;

        switch (value) {
            case 2:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_2_color));
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 4:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_4_color));
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 8:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_8_color));
                textColorId = R.color.text_tile_low; // Devrait être high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 16:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_16_color));
                textColorId = R.color.text_tile_low; // Devrait être high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 32:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_32_color));
                textColorId = R.color.text_tile_low; // Devrait être high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 64:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_64_color));
                textColorId = R.color.text_tile_low; // Devrait être high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 128:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_128_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 256:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_256_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 512:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_512_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 1024:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_1024_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large;
                break;
            case 2048:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_2048_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large;
                break;
            default:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_empty_color));
                textColorId = android.R.color.transparent;
                textSizeId = R.dimen.text_size_tile_small;
                if (value > 2048) {
                    tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_super_color));
                    textColorId = R.color.text_tile_high;
                    textSizeId = R.dimen.text_size_tile_large;
                }
                break;
        }

        if (value > 0) {
            tile.setText(String.valueOf(value));
            tile.setTextColor(ContextCompat.getColor(this, textColorId));
            tile.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(textSizeId));
        } else {
            tile.setText("");
        }
        tile.setGravity(Gravity.CENTER);
    }

    private void setupSwipeListener() {
        gameBoardLayout.setOnTouchListener(new OnSwipeTouchListener(MainActivity.this, new OnSwipeTouchListener.SwipeListener() {
            @Override public void onSwipeTop() { handleSwipe(Direction.UP); }
            @Override public void onSwipeBottom() { handleSwipe(Direction.DOWN); }
            @Override public void onSwipeLeft() { handleSwipe(Direction.LEFT); }
            @Override public void onSwipeRight() { handleSwipe(Direction.RIGHT); }
        }));
    }

    private void handleSwipe(Direction direction) {
        switch (direction) {
            case UP: game.pushUp(); break;
            case DOWN: game.pushDown(); break;
            case LEFT: game.pushLeft(); break;
            case RIGHT: game.pushRight(); break;
        }
        game.addNewNumbers();
        updateUI();
    }

    // Modifié pour montrer la dialog
    private void restartGame() {
        showRestartConfirmationDialog();
    }

    // Nouvelle méthode pour la dialog
    private void showRestartConfirmationDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_restart_confirm, null);
        builder.setView(dialogView);

        // Références vues dialog
        TextView dialogTitle = dialogView.findViewById(R.id.dialogTitle); // Non utilisé mais présent dans snippet
        TextView dialogMessage = dialogView.findViewById(R.id.dialogMessage); // Non utilisé mais présent dans snippet
        Button cancelButton = dialogView.findViewById(R.id.dialogCancelButton);
        Button confirmButton = dialogView.findViewById(R.id.dialogConfirmButton);

        final AlertDialog dialog = builder.create();

        cancelButton.setOnClickListener(v -> {
            dialog.dismiss();
        });

        confirmButton.setOnClickListener(v -> {
            dialog.dismiss();
            game = new Game(); // Création nouveau jeu
            game.addNewNumbers(); // Ajout tuiles initiales
            game.addNewNumbers();
            updateUI(); // Mise à jour UI
        });

        dialog.show();
    }


    private void showStats() {
        //A faire
    }
    private void showMenu() {
        //A faire
    }

    private void showMultiplayerScreen() {
        //A faire
    }

    // Ajout onPause
    @Override
    protected void onPause() {
        super.onPause();
        saveGame(); // Sauvegarde en quittant
    }

    // Nouvelle méthode sauvegarde
    private void saveGame() {
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString(GAME_STATE_KEY, game.serialize()); // Sauvegarde état sérialisé
        editor.putInt(HIGH_SCORE_KEY, game.getHighScore()); // Sauvegarde high score
        editor.apply(); // Appliquer
    }

    // Nouvelle méthode chargement
    private void loadGame() {
        String gameState = sharedPreferences.getString(GAME_STATE_KEY, null); // Récupère état
        if (gameState != null) {
            game = Game.deserialize(gameState); // Restaure si existe
            // game.loadHighScore(); // Pas dans le snippet loadGame
        } else {
            game = new Game(); // Nouveau jeu sinon
            game.addNewNumbers();
            game.addNewNumbers();
        }
        game.loadHighScore(); // Présent dans le snippet loadGame
        updateUI(); // MAJ UI

        // Logique high score séparée comme dans snippet
        int savedHighScore = sharedPreferences.getInt(HIGH_SCORE_KEY, 0);
        if (savedHighScore > game.getHighScore()) {
            game.setHighScore(savedHighScore);
            // updateUI(); // Pas de second updateUI dans le snippet ici
        }

    }

    private enum Direction {
        UP, DOWN, LEFT, RIGHT
    }

}